﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ForGet
{
   public class PackageRepository
   {
      static readonly string[] _supportedPackageExtenstions = new[] { "forpkg", "nupkg" };

      IEnumerable<string> _repos;

      public PackageRepository(IEnumerable<string> repos)
      {
         _repos = repos;
      }

      public IEnumerable<string> GetPackagesById(string id)
      {
         return
            from repo in _repos
            let repoPathforDependency = Path.Combine(repo, id)
            where Directory.Exists(repoPathforDependency)
               from extenstion in _supportedPackageExtenstions
                  from filePath in Directory.GetFiles(repoPathforDependency, string.Format("*.{0}", extenstion), SearchOption.AllDirectories)
                  select filePath;
      }

      public string GetPackage(string id, string version)
      {
         var matches =
            from package in GetPackagesById(id)
            let packageNameWithoutExtension = Path.GetFileNameWithoutExtension(package)
            where IsPackageMatch(package, id, version)
            select package;

         return matches.FirstOrDefault();
      }

      bool IsPackageMatch(string filename, string id, string version)
      {
         filename = Path.GetFileNameWithoutExtension(filename).ToLowerInvariant();

         if (!filename.StartsWith(id.ToLowerInvariant()))
            return false;
         
         Version expectedVersion;
         if (!TryParseVersion(version, out expectedVersion))
            return false;

         Version fileVersion;
         if (!TryParseVersion(filename.Substring(id.Length + 1), out fileVersion))
            return false;

         return fileVersion == expectedVersion;
      }

      bool TryParseVersion(string version, out Version result)
      {
         while (version.Count(c => c == '.') < 3)
         {
            version = version + ".0";
         }

         return Version.TryParse(version, out result);
      }
   }
}
